"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var prisma_datamodel_1 = require("prisma-datamodel");
var bson_1 = require("bson");
var ObjectTypeIdentifier = 'EmbeddedObject';
var MongoIdName = '_id';
var UnsupportedTypeErrorKey = 'UnsupportedType';
var UnsupportedArrayTypeErrorKey = 'UnsupportedArrayType';
var UnsupportedTypeError = /** @class */ (function (_super) {
    __extends(UnsupportedTypeError, _super);
    function UnsupportedTypeError(message, invalidType) {
        var _this = _super.call(this, message) || this;
        _this.message = message;
        _this.name = UnsupportedTypeErrorKey;
        _this.invalidType = invalidType;
        return _this;
    }
    return UnsupportedTypeError;
}(Error));
var UnsupportedArrayTypeError = /** @class */ (function (_super) {
    __extends(UnsupportedArrayTypeError, _super);
    function UnsupportedArrayTypeError(message, invalidType) {
        var _this = _super.call(this, message) || this;
        _this.message = message;
        _this.name = UnsupportedArrayTypeErrorKey;
        _this.invalidType = invalidType;
        return _this;
    }
    return UnsupportedArrayTypeError;
}(Error));
/**
 * Infers structure of a datatype from a set of data samples.
 */
var ModelMerger = /** @class */ (function () {
    function ModelMerger(name, isEmbedded) {
        if (isEmbedded === void 0) { isEmbedded = false; }
        this.name = name;
        this.isEmbedded = isEmbedded;
        this.fields = {};
        this.embeddedTypes = {};
    }
    ModelMerger.prototype.analyze = function (data) {
        var e_1, _a;
        try {
            for (var _b = __values(Object.keys(data)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var fieldName = _c.value;
                this.analyzeField(fieldName, data[fieldName]);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    /**
     * Gets the top level type only.
     */
    ModelMerger.prototype.getTopLevelType = function () {
        var _this = this;
        var fields = Object.keys(this.fields).map(function (key) { return _this.toIGQLField(_this.fields[key]); });
        return {
            fields: fields,
            isEmbedded: this.isEmbedded,
            name: this.name,
            isEnum: false // No enum in mongo
        };
    };
    /**
     * Gets the type with embedded types attached recursivley.
     */
    ModelMerger.prototype.getType = function () {
        var e_2, _a, e_3, _b;
        var type = this.getTopLevelType();
        var allEmbedded = [];
        try {
            for (var _c = __values(Object.keys(this.embeddedTypes)), _d = _c.next(); !_d.done; _d = _c.next()) {
                var embeddedFieldName = _d.value;
                var embedded = this.embeddedTypes[embeddedFieldName].getType();
                allEmbedded.push.apply(allEmbedded, __spread([embedded.type], embedded.embedded));
                try {
                    for (var _e = __values(type.fields), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var field = _f.value;
                        if (field.name === embeddedFieldName) {
                            field.type = embedded.type;
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return { type: type, embedded: allEmbedded };
    };
    // TODO: If we get more types to summarize, we should have all summarization code (e.g. for arrays) 
    // in one place.
    ModelMerger.prototype.summarizeTypeList = function (typeCandidates) {
        // Our float/int decision is based soley on the data itself.
        // If we have at least one float, integer is always a misguess.
        if (typeCandidates.indexOf(prisma_datamodel_1.TypeIdentifiers.float) >= 0) {
            typeCandidates = typeCandidates.filter(function (x) { return x !== prisma_datamodel_1.TypeIdentifiers.integer; });
        }
        return typeCandidates;
    };
    ModelMerger.prototype.toIGQLField = function (info) {
        var type = ModelMerger.ErrorType;
        var isArray = false;
        var isRequired = false;
        var comments = [];
        if (info.isArray.length > 1) {
            comments.push({
                isError: true,
                text: 'Datatype inconsistency: Sometimes is array, and sometimes not.'
            });
        }
        else if (info.isArray.length === 1) {
            isArray = info.isArray[0];
        }
        var typeCandidates = this.summarizeTypeList(__spread(info.types));
        if (typeCandidates.length > 1) {
            comments.push({
                isError: true,
                text: 'Datatype inconsistency. Conflicting types found: ' + typeCandidates.join(', ')
            });
        }
        if (typeCandidates.length === 1) {
            type = typeCandidates[0];
        }
        else {
            comments.push({
                isError: true,
                text: 'No type information found for field.'
            });
        }
        // https://www.prisma.io/docs/releases-and-maintenance/releases-and-beta-access/mongodb-preview-b6o5/#directives
        // TODO: we might want to include directives, as soon as we start changing names. 
        return {
            name: info.name,
            type: type,
            isId: info.name === MongoIdName,
            isList: isArray,
            isReadOnly: false,
            isRequired: isRequired,
            isUnique: false,
            relationName: null,
            relatedField: null,
            defaultValue: null,
            comments: comments
        };
    };
    ModelMerger.prototype.initField = function (name) {
        this.fields[name] = this.fields[name] || {
            invalidTypes: [],
            isArray: [],
            name: name,
            types: []
        };
    };
    ModelMerger.prototype.analyzeField = function (name, value) {
        try {
            var typeInfo = this.inferType(value);
            // Recursive embedding case. 
            if (typeInfo.type === ObjectTypeIdentifier) {
                this.embeddedTypes[name] = this.embeddedTypes[name] || new ModelMerger(prisma_datamodel_1.capitalize(name), true);
                this.embeddedTypes[name].analyze(value);
            }
            this.initField(name);
            this.fields[name] = this.mergeField(this.fields[name], typeInfo);
        }
        catch (err) {
            if (err.name === UnsupportedTypeErrorKey) {
                this.initField(name);
                this.fields[name].invalidTypes.push(err.invalidType);
            }
            else if (err.name == UnsupportedArrayTypeErrorKey) {
                this.initField(name);
                this.fields[name] = this.mergeField(this.fields[name], {
                    isArray: true,
                    type: null
                });
                this.fields[name].invalidTypes.push(err.invalidType);
            }
            else {
                throw err;
            }
        }
    };
    ModelMerger.prototype.mergeField = function (field, info) {
        var types = field.types;
        if (info.type !== null) {
            types = this.merge(field.types, info.type);
        }
        return {
            invalidTypes: field.invalidTypes,
            isArray: this.merge(field.isArray, info.isArray),
            name: field.name,
            types: types
        };
    };
    ModelMerger.prototype.merge = function (target, value) {
        if (target.indexOf(value) < 0) {
            target.push(value);
        }
        return target;
    };
    ModelMerger.prototype.inferArrayType = function (array) {
        var e_4, _a;
        var type = null;
        try {
            for (var array_1 = __values(array), array_1_1 = array_1.next(); !array_1_1.done; array_1_1 = array_1.next()) {
                var item = array_1_1.value;
                var itemType = this.inferType(item);
                if (itemType.isArray) {
                    throw new UnsupportedArrayTypeError('Received a nested array while analyzing data. This is not supported yet.', 'ArrayArray');
                }
                if (type === null) {
                    type = itemType.type;
                }
                else if (type === prisma_datamodel_1.TypeIdentifiers.integer && itemType.type === prisma_datamodel_1.TypeIdentifiers.float ||
                    type === prisma_datamodel_1.TypeIdentifiers.float && itemType.type === prisma_datamodel_1.TypeIdentifiers.integer) {
                    // Special case: We treat int as a case of float, if needed.
                    type = prisma_datamodel_1.TypeIdentifiers.float;
                }
                else if (type !== itemType.type) {
                    throw new UnsupportedArrayTypeError('Mixed arrays are not supported.', "Array of " + type + " and " + itemType.type);
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (array_1_1 && !array_1_1.done && (_a = array_1.return)) _a.call(array_1);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return type;
    };
    // TODO: There are more BSON types exported by the mongo client lib we could add here.
    ModelMerger.prototype.inferType = function (value) {
        // Maybe an array, which would otherwise be identified as object.
        if (Array.isArray(value)) {
            var arrayType = { type: this.inferArrayType(value), isArray: false };
            return { type: arrayType.type, isArray: true };
        }
        // Base types
        switch (typeof (value)) {
            case "number": return { type: value % 1 === 0 ? prisma_datamodel_1.TypeIdentifiers.integer : prisma_datamodel_1.TypeIdentifiers.float, isArray: false };
            case "boolean": return { type: prisma_datamodel_1.TypeIdentifiers.boolean, isArray: false };
            case "string": return { type: prisma_datamodel_1.TypeIdentifiers.string, isArray: false };
            case "object":
                if (value instanceof bson_1.ObjectID) {
                    return { type: prisma_datamodel_1.TypeIdentifiers.string, isArray: false };
                }
                else {
                    return { type: ObjectTypeIdentifier, isArray: false };
                }
            default: break;
        }
        throw new UnsupportedTypeError('Received an unsupported type:', typeof value);
    };
    ModelMerger.ErrorType = '<Unknown>';
    return ModelMerger;
}());
exports.ModelMerger = ModelMerger;
//# sourceMappingURL=modelMerger.js.map